"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongodb_1 = require("mongodb");
exports.buildDataContext = (client, dbName) => ((DBClient, DBName) => {
    let DBInstance;
    const resolveDB = () => new Promise((resolve, reject) => {
        if (DBInstance === undefined) {
            DBClient.connect()
                .then(() => {
                DBInstance = DBClient.db(DBName);
                resolve(DBInstance);
            })
                .catch((error) => {
                reject(error);
            });
        }
        else {
            resolve(DBInstance);
        }
    });
    const resolveCollection = async (collection) => {
        const db = await resolveDB();
        return db.collection(collection);
    };
    const dataContext = {
        close: () => DBClient.close(true),
        createItem: async (collection, item, options = {}) => {
            const Collection = await resolveCollection(collection);
            const { result } = await Collection.insertOne(item, options);
            return { affected: result.n };
        },
        db: resolveDB,
        newID: () => new mongodb_1.ObjectId(),
        queryByID: async (collection, id, options = {}) => {
            const Collection = await resolveCollection(collection);
            const filter = { _id: id };
            const item = await Collection.findOne(filter, options);
            if (item === undefined || item === null) {
                return undefined;
            }
            else {
                return item;
            }
        },
        removeByID: async (collection, id, options = {}) => {
            const Collection = await resolveCollection(collection);
            const { result } = await Collection.deleteOne({ _id: id }, options);
            return { affected: result.n };
        },
        toRepository: (collection, defaultOptions = {}) => ({
            collection: () => resolveCollection(collection),
            createItem: (item, options = {}) => dataContext.createItem(collection, item, Object.assign(options, defaultOptions)),
            newID: () => dataContext.newID(),
            queryByID: (id, options = {}) => dataContext.queryByID(collection, id, Object.assign(options, defaultOptions)),
            removeByID: (id, options = {}) => dataContext.removeByID(collection, id, Object.assign(options, defaultOptions)),
        }),
    };
    return dataContext;
})(client, dbName);
Object.freeze(exports.buildDataContext);
//# sourceMappingURL=buildDataContext.js.map